section .data
global page_directory
extern os_alloc_page

align 4096
page_directory:
    times 1024 dd 0 ; Page directory table

align 4096
page_table:
    times 1024 dd 0 ; First page table

section .text
global setup_paging
global alloc_page_table

setup_paging:
    ; Initialize the first page table - mapping the first 4MB of memory
    mov ecx, 0
    mov edx, page_table
    mov eax, 0x3 ; Present + Writable
.map_pt:
    mov [edx + ecx * 4], eax
    add eax, 0x1000 ; Next page physical address
    inc ecx
    cmp ecx, 1024 ; Map 1024 pages (4MB)
    jl .map_pt

    ; Set the page directory entry
    mov eax, page_table
    or eax, 0x3 ; Present + Writable
    mov [page_directory], eax

    ; Load the page directory address into CR3
    mov eax, page_directory
    mov cr3, eax

    ; Enable paging by setting the PG bit in CR0
    mov eax, cr0
    or eax, 0x80000000
    mov cr0, eax

    ret

alloc_page_table:
    ; Allocate a new page for the page table
    call os_alloc_page ; Assume os_alloc_page function is implemented in C
    mov [eax], dword 0 ; Clear the page table
    ret