#ifndef IO_H
#define IO_H

#define PIT_CONTROL_PORT 0x43
#define PIT_CHANNEL_0_PORT 0x40

// PIT的输入时钟频率
#define PIT_INPUT_FREQUENCY 1193180

// 期望的时钟中断频率
#define DESIRED_TICKS_PER_SECOND 100

// 计算LATCH值，即每个时钟周期计数器需要倒数的次数
#define LATCH (PIT_INPUT_FREQUENCY / DESIRED_TICKS_PER_SECOND)

// 向端口写入一个字节
static inline void outb(unsigned short port, unsigned char value)
{
    asm volatile("outb %0, %1" : : "a"(value), "Nd"(port));
}

// 从端口读取一个字节
static inline unsigned char inb(unsigned short port)
{
    unsigned char result;
    asm volatile("inb %1, %0" : "=a"(result) : "Nd"(port));
    return result;
}

// 向端口写入一个字（2字节）
static inline void outw(unsigned short port, unsigned short value)
{
    asm volatile("outw %0, %1" : : "a"(value), "Nd"(port));
}

// 从端口读取一个字（2字节）
static inline unsigned short inw(unsigned short port)
{
    unsigned short result;
    asm volatile("inw %1, %0" : "=a"(result) : "Nd"(port));
    return result;
}

// 向端口写入一个双字（4字节）
static inline void outl(unsigned short port, unsigned int value)
{
    asm volatile("outl %0, %1" : : "a"(value), "Nd"(port));
}

// 从端口读取一个双字（4字节）
static inline unsigned int inl(unsigned short port)
{
    unsigned int result;
    asm volatile("inl %1, %0" : "=a"(result) : "Nd"(port));
    return result;
}

static inline void io_wait(void) { /*未使用端口*/ outb(0x80, 0); }

#endif // IO_H
