#ifndef KERNEL_H
#define KERNEL_H

#include "types.h"
#include "video.h"
#include "interrupt.h"
#include "memory.h"
#include "sched.h"
#include "syscall.h"
#include "io.h"
// Keyboard port numbers
#define KEYBOARD_DATA_PORT 0x60
#define KEYBOARD_STATUS_PORT 0x64

// Keyboard port numbers
#define KEYBOARD_DATA_PORT 0x60
#define KEYBOARD_STATUS_PORT 0x64

// Keyboard status bits
#define KEYBOARD_OUTPUT_FULL 0x01 // Output buffer full

#define KEY_SHIFT 0x2A         // Left shift scancode
#define KEY_SHIFT_RELEASE 0xAA // Left shift release scancode
// Function Declarations
void get_input(char *buffer, int max_len);
int eval_expression(const char *expr);
void print_int(int num);
char getchar();
void putchar(char c);
char scancode_to_ascii(unsigned char scancode, int shift_pressed);

// Kernel Initialization
void kmain(void);                // Kernel main function

// System Initialization
void clear_screen(void);         // Clear the video memory

// System Services
void log_message(const char *msg);   // Log a message to the screen
void print_int(int num);            // Print an integer to the screen

// System Time and Scheduling
void schedule(void);               // Process scheduling function
void timer_interrupt_handler(void); // Timer interrupt handler for scheduling
void init_scheduler(void);         // Initialize the scheduler

#endif // KERNEL_H
