#ifndef MEMORY_H
#define MEMORY_H

#include "types.h"
#include "utils.h"

// PTE FLAGS
#define PTE_PRESENT  0x1      // Page is present in memory
#define PTE_WRITE    0x2      // Page is writable
#define PTE_USER     0x4      // User-level access
#define PTE_PWT      0x8      // Page-level Write-through
#define PTE_PCD      0x10     // Page-level Cache Disable
#define PTE_ACCESSED 0x20     // Page has been accessed
#define PTE_DIRTY    0x40     // Page has been written to
#define PTE_ATTRIB   0x80     // Additional attributes
#define MAX_TIME 0xFFFFFFFFFFFFFFFFULL
#define PAGE_LOCKED (1ULL << 63) // 使用最高位表示页面被锁定

#define PAGE_SIZE 0X1000

// External assembly function to set up paging
extern void setup_paging(void);

// Memory management functions
void memory_init(void); // Initialize memory system
void* alloc_page(void); // Allocate a single physical page
void *os_alloc_page(void);
void free_page(void* ptr); // Free a previously allocated page

// LRU support functions
void update_page_access_time(uint64_t page_index);  // Update the access time of a page
void lru_evict_page(void);  // Evict the least recently used page

// Bitmap or linked list for tracking free pages
extern uint64_t *free_page_bitmap;
extern uint64_t total_pages; // Total number of pages in memory

// Array for tracking the last accessed time of pages
extern uint64_t *page_access_time;

#endif /* MEMORY_H */
