#ifndef SYSCALL_H
#define SYSCALL_H

#include "types.h"

#define GET_REGISTER(reg, var) \
    asm volatile(GET_##reg : "=r" (var))

// System Call Numbers
#define SYSCALL_WRITE    0
#define SYSCALL_EXIT     1
#define SYSCALL_READ     2
#define SYSCALL_ALLOC    3
#define SYSCALL_FREE     4
#define SYSCALL_SLEEP    5

// Function prototypes
void syscall_handler(void);    // The function that handles syscalls

// System call implementations
void syscall_write(const char *str);
void syscall_exit(int status);
void syscall_read(char *buffer, size_t size);
void* syscall_alloc();
void syscall_free(void* ptr);
void syscall_sleep(int milliseconds);

// System tick related functions
unsigned int get_system_ticks(void);  // Returns the current tick count
#endif // SYSCALL_H
