#include "video.h"
#include "string.h"
#include "utils.h"
#include "syscall.h"
#include "kernel.h"
#include "memory.h"

int clock_command = 0;

void kmain(void)
{
    clear_screen();
    log_message("Bootloader started kernel successfully.");
    memory_init();
    log_message("Memory system initialized.");
    init_thread_queue();
    log_message("Thread Queue initialized.");
    init_interrupts();
    log_message("Interrupt system initialized.");
    log_message("Welcome to YuMe OS.(Author: Xu Pan, Yu Liang and Yuhao Fang of HFUT)");
    while (1)
    {
        print_string("> ");

        char input[100];
        get_input(input, 100);
        if (strcmp(input, "hello") == 0)
        {
            log_message("Hello YuMe OS!");
            continue; 
        }
        else if (strcmp(input, "help") == 0)
        {
            log_message("Available commands: ");
            log_message("1. hello - Screen output test");
            log_message("2. echo - Say what you say");
            log_message("3. <expression> - Evaluate a mathematical expression");
            continue; 
        }
        else if (strcmp(input, "clear") == 0)
        {
            clear_screen();
            continue;
        }
        else if (strncmp(input, "count ", 6) == 0)
        {
            clock_command = atoi(input + 6);
            io_wait();
            if(clock_command == 0){
                continue;
            }
        }
        else if (strncmp(input, "echo ", 5) == 0)
        {
            // Echo the input after the "echo " command
            print_string("\nYuMe: ");
            print_string(input + 5); // Print everything after "echo "
            print_string("\n");
            continue;
        }
        // Parse and evaluate the expression
        int result = eval_expression(input);

        // Print the result
        print_string("\nYuMe: ");
        print_int(result);
        print_string("\n");
    }
}

// Function to read user input (simple version)
void get_input(char *buffer, int max_len)
{
    int i = 0;
    char c;

    // Read characters until Enter (or max_len - 1)
    while (i < max_len - 1)
    {
        c = getchar(); // Get a character from the terminal

        if (c == '\n')
        {          // Newline character (Enter)
            break; // Stop reading when Enter is pressed
        }

        print_char(c); // Use print_char to display the input character

        buffer[i++] = c;
    }

    buffer[i] = '\0';
}

void print_int(int num)
{
    char str[20];
    itoa(num, str, 10); // Convert integer to string
    print_string(str);  // Print the string
}

char getchar(void)
{
    static int shift_pressed = 0;
    unsigned char scancode, ascii;

    while (1)
    {
        while ((inb(KEYBOARD_STATUS_PORT) & KEYBOARD_OUTPUT_FULL) == 0)
            ;
        scancode = inb(KEYBOARD_DATA_PORT);
        if (scancode == KEY_SHIFT)
        {
            shift_pressed = 1;
            continue;
        }
        else if (scancode == KEY_SHIFT_RELEASE)
        {
            shift_pressed = 0;
            continue;
        }
        if (!(scancode & 0x80))
        {
            ascii = scancode_to_ascii(scancode, shift_pressed);
            if (ascii)
            {
                return ascii;
            }
        }
    }
}

// Expanded scancode to ASCII conversion with shift support
char scancode_to_ascii(unsigned char scancode, int shift_pressed)
{
    // Regular (unshifted) characters
    static char ascii_table[] = {
        0, 0, '1', '2', '3', '4', '5', '6', '7', '8', '9', '0', '-', '=', 0,
        0, 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', '[', ']', '\n',
        0, 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', ';', '\'', '`',
        0, '\\', 'z', 'x', 'c', 'v', 'b', 'n', 'm', ',', '.', '/', 0,
        '*', 0, ' '};

    // Shifted characters
    static char ascii_table_shift[] = {0, 0, '!', '@', '#', '$', '%', '^', '&',
                                       '*', '(', ')', '_', '+', 0, 0, 'Q', 'W', 'E', 'R', 'T', 'Y', 'U', 'I', 'O',
                                       'P', '{', '}', '\n', 0, 'A', 'S', 'D', 'F', 'G', 'H', 'J', 'K', 'L', ':',
                                       '"', '~', 0, '|', 'Z', 'X', 'C', 'V', 'B', 'N', 'M', '<', '>', '?', 0,
                                       '*', 0, ' '};
    if (scancode < sizeof(ascii_table))
    {
        return shift_pressed ? ascii_table_shift[scancode] : ascii_table[scancode];
    }
    return 0;
}