#include "string.h"

// Calculate the length of a string
size_t strlen(const char *str) {
    size_t length = 0;
    while (str[length] != '\0') {
        length++;
    }
    return length;
}

// Compare two strings (returns 0 if equal, non-zero otherwise)
int strcmp(const char *str1, const char *str2) {
    while (*str1 && (*str1 == *str2)) {
        str1++;
        str2++;
    }
    return *(unsigned char *)str1 - *(unsigned char *)str2;
}

int strncmp(const char *s1, const char *s2, size_t n)
{
    while (n--)
    {
        // Compare the characters at the current positions
        if (*s1 != *s2)
        {
            return (unsigned char)*s1 - (unsigned char)*s2;
        }

        // If the characters are the same, move to the next character in both strings
        if (*s1 == '\0') // End of string reached
        {
            return 0; // If both strings end at the same point, they're equal
        }

        s1++;
        s2++;
    }

    return 0; // If `n` characters are compared and no difference found, return 0
}

// Copy the source string into the destination buffer
char *strcpy(char *dest, const char *src) {
    char *d = dest;
    while ((*d++ = *src++) != '\0');
    return dest;
}

// Concatenate the source string to the destination string
char *strcat(char *dest, const char *src) {
    char *d = dest;
    while (*d) {  // Move to the end of dest string
        d++;
    }
    while ((*d++ = *src++) != '\0');  // Copy src to dest
    return dest;
}

// Convert an integer to a string (itoa)
void itoa(int num, char *str, int base) {
    int i = 0;
    int isNegative = 0;

    // Handle 0 explicitly, otherwise empty string is printed
    if (num == 0) {
        str[i++] = '0';
        str[i] = '\0';
        return;
    }

    // Handle negative numbers only if the base is 10
    if (num < 0 && base == 10) {
        isNegative = 1;
        num = -num;
    }

    // Process individual digits
    while (num != 0) {
        int rem = num % base;
        str[i++] = (rem > 9) ? (rem - 10) + 'a' : rem + '0';
        num = num / base;
    }

    // Append negative sign for negative numbers
    if (isNegative) {
        str[i++] = '-';
    }

    str[i] = '\0';  // Null-terminate the string

    // Reverse the string (itoa produces the string in reverse order)
    reverse(str, i);
}

int atoi(const char *str) {
    int result = 0;      // Holds the result
    int sign = 1;        // 1 for positive, -1 for negative
    int i = 0;

    // Skip leading whitespaces
    while (str[i] == ' ') {
        i++;
    }

    // Check for an optional sign
    if (str[i] == '-') {
        sign = -1;
        i++;
    } else if (str[i] == '+') {
        i++;  // Skip the '+' sign
    }

    // Convert the string digits to an integer
    while (str[i] >= '0' && str[i] <= '9') {
        result = result * 10 + (str[i] - '0'); // Convert char to digit and add to result
        i++;
    }

    // Apply the sign
    return sign * result;
}

// Reverse a string in place
void reverse(char *str, size_t length) {
    size_t start = 0;
    size_t end = length - 1;
    while (start < end) {
        char temp = str[start];
        str[start] = str[end];
        str[end] = temp;
        start++;
        end--;
    }
}

void *memset(void *ptr, int value, size_t num)
{
    unsigned char *p = ptr;
    while (num--)
    {
        *p++ = (unsigned char)value;
    }
    return ptr;
}