#include "utils.h"
#include "video.h"
#include "string.h"

// Log a message to the screen (helper for debugging)
void log_message(const char *msg)
{
    print_string("\n[LOG] ");
    print_string(msg);
    print_string("\n");
}

// Get the absolute value of an integer
int abs_value(int num)
{
    if (num < 0)
    {
        return -num;
    }
    return num;
}

// Calculate base^exp (power function)
unsigned int power(unsigned int base, unsigned int exp)
{
    unsigned int result = 1;
    while (exp > 0)
    {
        if (exp % 2 == 1)
        {
            result *= base;
        }
        base *= base;
        exp /= 2;
    }
    return result;
}

// Swap two integers
void swap(int *a, int *b)
{
    int temp = *a;
    *a = *b;
    *b = temp;
}

// Simple delay function (not precise, but works for testing)
void delay(int count)
{
    volatile int i;
    for (i = 0; i < count; i++)
    {
        // Do nothing, just waste time
    }
}

// Convert an integer to hexadecimal string (e.g., "0x1F")
void itoa_hex(int num, char *str)
{
    int i = 0;
    int isNegative = 0;
    if (num < 0)
    {
        isNegative = 1;
        num = -num;
    }

    while (num != 0)
    {
        int rem = num % 16;
        str[i++] = (rem > 9) ? (rem - 10) + 'a' : rem + '0';
        num = num / 16;
    }

    if (isNegative)
    {
        str[i++] = '-';
    }

    // Add the "0x" prefix for hexadecimal
    str[i++] = 'x';
    str[i++] = '0';

    str[i] = '\0'; // Null terminate the string

    // Reverse the string (itoa produces the string in reverse order)
    reverse(str, i);
}

unsigned int hex_to_int(const char *hex_str)
{
    unsigned int result = 0;
    while (*hex_str)
    {
        result *= 16;
        if (*hex_str >= '0' && *hex_str <= '9')
        {
            result += *hex_str - '0';
        }
        else if (*hex_str >= 'a' && *hex_str <= 'f')
        {
            result += *hex_str - 'a' + 10;
        }
        else if (*hex_str >= 'A' && *hex_str <= 'F')
        {
            result += *hex_str - 'A' + 10;
        }
        hex_str++;
    }
    return result;
}

// Simple function to evaluate basic expressions
int eval_expression(const char *expr)
{
    int num1 = 0, num2 = 0;
    char operator= 0;
    int result = 0;
    int i = 0;

    // Skip leading spaces
    while (expr[i] == ' ')
    {
        i++;
    }

    // Parse the first number
    while (expr[i] >= '0' && expr[i] <= '9')
    {
        num1 = num1 * 10 + (expr[i] - '0');
        i++;
    }

    // Skip any spaces after the first number
    while (expr[i] == ' ')
    {
        i++;
    }

    // Check if the next character is an operator
    if (expr[i] == '+' || expr[i] == '-' || expr[i] == '*' || expr[i] == '/')
    {
        operator= expr[i];
        i++; // Move past the operator
    }
    else
    {
        log_message("Error: No operator found, or invalid instruction!");
        return 0;
    }

    // Skip spaces between operator and second number
    while (expr[i] == ' ')
    {
        i++;
    }

    // Parse the second number
    while (expr[i] >= '0' && expr[i] <= '9')
    {
        num2 = num2 * 10 + (expr[i] - '0');
        i++;
    }

    // Perform the operation based on the operator
    switch (operator)
    {
    case '+':
        result = num1 + num2;
        break;
    case '-':
        result = num1 - num2;
        break;
    case '*':
        result = num1 * num2;
        break;
    case '/':
        if (num2 != 0)
        {
            result = num1 / num2;
        }
        else
        {
            log_message("Error: Division by zero.");
            result = 0; // Return 0 in case of division by zero
        }
        break;
    default:
        log_message("Error: Unsupported operator.");
        result = 0;
        break;
    }

    return result;
}

int ffs_custom(uint64_t x) {
    if (x == 0) {
        return 0;  // No bits set
    }
    int index = 1;
    while ((x & 1) == 0) {
        x >>= 1;  // Shift the bits right by 1
        index++;
    }
    return index;
}

int max(int x, int y){
    return x >= y ? x : y;
}

int min(int x, int y){
    return x <= y ? x : y;
}